﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;

namespace Spiel
{
    abstract class Spielobjekt
    {
        static double breiteSpielfeld;
        static double höheSpielfeld;
        static protected double BreiteSpielfeld
        { get { return breiteSpielfeld; } }
        static protected double HöheSpielfeld
        { get { return höheSpielfeld; } }

        protected double x, y;
        public double X
        { get { return x; } }
        public double Y
        { get { return y; } }

        protected double vx, vy;
        public double Vx
        { get { return vx; } }
        public double Vy
        { get { return vy; } }

        public Spielobjekt(double x, double y, double vx, double vy)
        {
            this.x = x;
            this.y = y;
            this.vx = vx;
            this.vy = vy;
        }

        public static void SetzeAbmessungenSpielfeld(double breite, double höhe)
        {
            Spielobjekt.breiteSpielfeld = breite;
            Spielobjekt.höheSpielfeld = höhe;
        }

        public virtual void Bewege(double zeitschritt)
        {
            x += vx * zeitschritt;
            y += vy * zeitschritt;
        }

        public void BringeAufZeichenflächeZurück()
        {
            x -= BreiteSpielfeld * Math.Floor(x / BreiteSpielfeld);
            y -= HöheSpielfeld * Math.Floor(y / HöheSpielfeld);
        }

        public abstract void Zeichne(Canvas c);
    }

    class Raumschiff : Spielobjekt
    {
        double winkel = -90.0;
        Polygon polygon = new Polygon();

        public double Winkel
        { get { return winkel; } }

        public Raumschiff()
            : base(0.5 * BreiteSpielfeld, 0.5 * HöheSpielfeld, 0.0, 0.0)
        {
            polygon.Points.Add(new Point(10.0, 0.0));
            polygon.Points.Add(new Point(-10.0, -4.0));
            polygon.Points.Add(new Point(-10.0, 4.0));
            polygon.Fill = Brushes.Blue;
        }

        public override void Zeichne(Canvas c)
        {
            polygon.RenderTransform = new RotateTransform(winkel);
            c.Children.Add(polygon);
            Canvas.SetLeft(polygon, x);
            Canvas.SetTop(polygon, y);
        }

        public void Drehe(bool istImUhrzeigersinn)
        {
            winkel += istImUhrzeigersinn ? 10.0 : -10.0;
            double v = Math.Sqrt(vx * vx + vy * vy);
            double winkelInRadiant = Math.PI * winkel / 180.0;
            vx = v * Math.Cos(winkelInRadiant);
            vy = v * Math.Sin(winkelInRadiant);
        }

        internal void Beschleunige(bool beschleunige)
        {
            double winkelInRadiant = Math.PI * winkel / 180.0;
            if(beschleunige)
            {
                vx = 1.05 * vx + 0.5 * Math.Cos(winkelInRadiant);
                vy = 1.05 * vy + 0.5 * Math.Sin(winkelInRadiant);
            }
            else
            {
                vx *= 0.9;
                vy *= 0.9;
            }
        }
    }

    class Asteroid : Spielobjekt
    {
        static Random würfel = new Random();
        Polygon polygon = new Polygon();

        public Asteroid(double radius, double x, double y, double vx, double vy)
            : base(x, y, vx, vy)
        {
            for (int i = 0; i < 12; i++)
            {
                double w = Math.PI * i / 6.0;
                double r = radius * (0.8 + 0.4 * würfel.NextDouble());
                polygon.Points.Add(new Point(r * Math.Cos(w), r * Math.Sin(w)));
            }
            polygon.Fill = Brushes.Gray;
        }

        public static Asteroid ErzeugeAmRand()
        {
            double radius = 20.0 + 10.0 * würfel.NextDouble();
            int welcherRand = würfel.Next(4);
            double position = würfel.NextDouble();
            double richtung = Math.PI * würfel.NextDouble(); // nur 0°...180°
            double va = 50.0 * Math.Cos(richtung);
            double vb = 50.0 * Math.Sin(richtung);
            Asteroid a = null;
            switch(welcherRand)
            {
                case 0: // links
                    a = new Asteroid(radius, 0.0, position * HöheSpielfeld, vb, va);
                    break;
                case 1: // oben
                    a = new Asteroid(radius, position * BreiteSpielfeld, 0.0, va, vb);
                    break;
                case 2: // rechts
                    a = new Asteroid(radius, BreiteSpielfeld, position * HöheSpielfeld, -vb, va);
                    break;
                case 3: // unten
                    a = new Asteroid(radius, position * BreiteSpielfeld, HöheSpielfeld, va, -vb);
                    break;
            }
            return a;
        }

        public override void Zeichne(Canvas c)
        {
            c.Children.Add(polygon);
            Canvas.SetLeft(polygon, x);
            Canvas.SetTop(polygon, y);
        }

        public bool EnthältPunkt(double x, double y)
        {
            return polygon.RenderedGeometry.FillContains(new Point(x - this.x, y - this.y));
        }
    }

    class Photonentorpedo : Spielobjekt
    {
        Ellipse ellipse = new Ellipse();

        public Photonentorpedo(double x, double y, double vx, double vy, double winkel)
            : base(x, y, vx + 30.0 * Math.Cos(Math.PI * winkel / 180.0), vy + 30.0 * Math.Sin(Math.PI * winkel / 180.0))
        {
            ellipse.Fill = Brushes.Red;
            ellipse.Width = 4.0;
            ellipse.Height = 4.0;
        }

        public override void Zeichne(Canvas c)
        {
            c.Children.Add(ellipse);
            Canvas.SetLeft(ellipse, x - 0.5 * ellipse.Width);
            Canvas.SetTop(ellipse, y - 0.5 * ellipse.Height);
        }

        public bool HatSpielfeldVerlassen()
        {
            return x < 0.0 || y < 0.0 || x > BreiteSpielfeld || y > HöheSpielfeld;
        }
    }
}
